
    create table gr_gruser (
        id int8 not null,
        dateCreation timestamp,
        emailAddress varchar(255),
        enabled bool not null,
        extId varchar(255) unique,
        fullName varchar(255),
        name varchar(255) not null unique,
        password varchar(255),
        primary key (id)
    );

    create table gr_gsinstance (
        id int8 not null,
        baseURL varchar(255) not null,
        dateCreation timestamp,
        description varchar(255),
        name varchar(255) not null,
        password varchar(255) not null,
        username varchar(255) not null,
        primary key (id)
    );

    create table gr_gsuser (
        id int8 not null,
        admin bool not null,
        allowedArea geometry,
        dateCreation timestamp,
        emailAddress varchar(255),
        enabled bool not null,
        extId varchar(255) unique,
        fullName varchar(255),
        name varchar(255) not null unique,
        password varchar(255),
        profile_id int8 not null,
        primary key (id)
    );

    create table gr_layer_attributes (
        details_id int8 not null,
        access_type varchar(255),
        data_type varchar(255),
        name varchar(255) not null,
        primary key (details_id, name),
        unique (details_id, name)
    );

    create table gr_layer_custom_props (
        details_id int8 not null,
        propvalue varchar(255),
        propkey varchar(255),
        primary key (details_id, propkey)
    );

    create table gr_layer_details (
        id int8 not null,
        area geometry,
        cqlFilterRead varchar(4000),
        cqlFilterWrite varchar(4000),
        defaultStyle varchar(255),
        type varchar(255),
        rule_id int8 not null,
        primary key (id),
        unique (rule_id)
    );

    create table gr_layer_styles (
        details_id int8 not null,
        styleName varchar(255)
    );

    create table gr_profile (
        id int8 not null,
        dateCreation timestamp,
        enabled bool not null,
        extId varchar(255) unique,
        name varchar(255) not null unique,
        primary key (id)
    );

    create table gr_profile_custom_props (
        profile_id int8 not null,
        propvalue varchar(255),
        propkey varchar(255),
        primary key (profile_id, propkey)
    );

    create table gr_rule (
        id int8 not null,
        grant_type varchar(255) not null,
        layer varchar(255),
        priority int8 not null,
        request varchar(255),
        service varchar(255),
        workspace varchar(255),
        gsuser_id int8,
        instance_id int8,
        profile_id int8,
        primary key (id),
        unique (gsuser_id, profile_id, instance_id, service, request, workspace, layer)
    );

    create table gr_rule_limits (
        id int8 not null,
        area geometry,
        rule_id int8 not null,
        primary key (id),
        unique (rule_id)
    );

    alter table gr_gsuser 
        add constraint fk_user_profile 
        foreign key (profile_id) 
        references gr_profile;

    alter table gr_layer_attributes 
        add constraint fk_attribute_layer 
        foreign key (details_id) 
        references gr_layer_details;

    alter table gr_layer_custom_props 
        add constraint fk_custom_layer 
        foreign key (details_id) 
        references gr_layer_details;

    alter table gr_layer_details 
        add constraint fk_details_rule 
        foreign key (rule_id) 
        references gr_rule;

    alter table gr_layer_styles 
        add constraint fk_styles_layer 
        foreign key (details_id) 
        references gr_layer_details;

    alter table gr_profile_custom_props 
        add constraint fk_custom_profile 
        foreign key (profile_id) 
        references gr_profile;

    create index idx_rule_request on gr_rule (request);

    create index idx_rule_layer on gr_rule (layer);

    create index idx_rule_service on gr_rule (service);

    create index idx_rule_workspace on gr_rule (workspace);

    create index idx_rule_priority on gr_rule (priority);

    alter table gr_rule 
        add constraint fk_rule_user 
        foreign key (gsuser_id) 
        references gr_gsuser;

    alter table gr_rule 
        add constraint fk_rule_profile 
        foreign key (profile_id) 
        references gr_profile;

    alter table gr_rule 
        add constraint fk_rule_instance 
        foreign key (instance_id) 
        references gr_gsinstance;

    alter table gr_rule_limits 
        add constraint fk_limits_rule 
        foreign key (rule_id) 
        references gr_rule;

    create sequence hibernate_sequence;
